"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, HttpClient
from .sdkconfiguration import SDKConfiguration
from .utils.retries import RetryConfig
from clerk_dev import models
from clerk_dev._hooks import SDKHooks
from clerk_dev.actortokens import ActorTokens
from clerk_dev.allowlistidentifiers import AllowlistIdentifiers
from clerk_dev.betafeatures import BetaFeatures
from clerk_dev.blocklistidentifiers_sdk import BlocklistIdentifiersSDK
from clerk_dev.clients import Clients
from clerk_dev.domains_sdk import DomainsSDK
from clerk_dev.emailaddresses import EmailAddresses
from clerk_dev.instancesettings_sdk import InstanceSettingsSDK
from clerk_dev.invitations import Invitations
from clerk_dev.jwks import Jwks
from clerk_dev.jwttemplates import JwtTemplates
from clerk_dev.misc import Misc
from clerk_dev.oauthapplications_sdk import OAuthApplicationsSDK
from clerk_dev.organizationinvitations_sdk import OrganizationInvitationsSDK
from clerk_dev.organizationmemberships_sdk import OrganizationMembershipsSDK
from clerk_dev.organizations_sdk import OrganizationsSDK
from clerk_dev.phonenumbers import PhoneNumbers
from clerk_dev.proxychecks import ProxyChecks
from clerk_dev.redirecturls import RedirectUrls
from clerk_dev.samlconnections_sdk import SamlConnectionsSDK
from clerk_dev.sessions import Sessions
from clerk_dev.signintokens import SignInTokens
from clerk_dev.signups import SignUps
from clerk_dev.templates import Templates
from clerk_dev.testingtokens import TestingTokens
from clerk_dev.types import Nullable, UNSET
from clerk_dev.users import Users
import clerk_dev.utils as utils
from clerk_dev.webhooks import Webhooks
import httpx
from typing import Callable, Dict, Optional, Union

class Clerk(BaseSDK):
    r"""Clerk Backend API: The Clerk REST Backend API, meant to be accessed by backend
    servers.

    ### Versions

    When the API changes in a way that isn't compatible with older versions, a new version is released.
    Each version is identified by its release date, e.g. `2021-02-05`. For more information, please see [Clerk API Versions](https://clerk.com/docs/backend-requests/versioning/overview).


    Please see https://clerk.com/docs for more information.
    https://clerk.com/docs
    """
    actor_tokens: ActorTokens
    allowlist_identifiers: AllowlistIdentifiers
    beta_features: BetaFeatures
    blocklist_identifiers: BlocklistIdentifiersSDK
    clients: Clients
    domains: DomainsSDK
    email_addresses: EmailAddresses
    instance_settings: InstanceSettingsSDK
    invitations: Invitations
    jwks: Jwks
    jwt_templates: JwtTemplates
    o_auth_applications: OAuthApplicationsSDK
    organizations: OrganizationsSDK
    organization_invitations: OrganizationInvitationsSDK
    organization_memberships: OrganizationMembershipsSDK
    phone_numbers: PhoneNumbers
    proxy_checks: ProxyChecks
    misc: Misc
    redirect_urls: RedirectUrls
    saml_connections: SamlConnectionsSDK
    sessions: Sessions
    sign_in_tokens: SignInTokens
    sign_ups: SignUps
    templates: Templates
    testing_tokens: TestingTokens
    users: Users
    webhooks: Webhooks
    def __init__(
        self,
        bearer_auth: Union[str, Callable[[], str]],
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: Optional[Nullable[RetryConfig]] = UNSET
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param bearer_auth: The bearer_auth required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        """
        if client is None:
            client = httpx.Client()

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        if async_client is None:
            async_client = httpx.AsyncClient()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security = None
        if callable(bearer_auth):
            security = lambda: models.Security(bearer_auth = bearer_auth()) # pylint: disable=unnecessary-lambda-assignment
        else:
            security = models.Security(bearer_auth = bearer_auth)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)
    

        BaseSDK.__init__(self, SDKConfiguration(
            client=client,
            async_client=async_client,
            security=security,
            server_url=server_url,
            server_idx=server_idx,
            retry_config=retry_config
        ))

        hooks = SDKHooks()

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(current_server_url, self.sdk_configuration.client)
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self._init_sdks()


    def _init_sdks(self):
        self.actor_tokens = ActorTokens(self.sdk_configuration)
        self.allowlist_identifiers = AllowlistIdentifiers(self.sdk_configuration)
        self.beta_features = BetaFeatures(self.sdk_configuration)
        self.blocklist_identifiers = BlocklistIdentifiersSDK(self.sdk_configuration)
        self.clients = Clients(self.sdk_configuration)
        self.domains = DomainsSDK(self.sdk_configuration)
        self.email_addresses = EmailAddresses(self.sdk_configuration)
        self.instance_settings = InstanceSettingsSDK(self.sdk_configuration)
        self.invitations = Invitations(self.sdk_configuration)
        self.jwks = Jwks(self.sdk_configuration)
        self.jwt_templates = JwtTemplates(self.sdk_configuration)
        self.o_auth_applications = OAuthApplicationsSDK(self.sdk_configuration)
        self.organizations = OrganizationsSDK(self.sdk_configuration)
        self.organization_invitations = OrganizationInvitationsSDK(self.sdk_configuration)
        self.organization_memberships = OrganizationMembershipsSDK(self.sdk_configuration)
        self.phone_numbers = PhoneNumbers(self.sdk_configuration)
        self.proxy_checks = ProxyChecks(self.sdk_configuration)
        self.misc = Misc(self.sdk_configuration)
        self.redirect_urls = RedirectUrls(self.sdk_configuration)
        self.saml_connections = SamlConnectionsSDK(self.sdk_configuration)
        self.sessions = Sessions(self.sdk_configuration)
        self.sign_in_tokens = SignInTokens(self.sdk_configuration)
        self.sign_ups = SignUps(self.sdk_configuration)
        self.templates = Templates(self.sdk_configuration)
        self.testing_tokens = TestingTokens(self.sdk_configuration)
        self.users = Users(self.sdk_configuration)
        self.webhooks = Webhooks(self.sdk_configuration)
    
